@@ -1,7 +1,7 @@
+from types import UnionType
 from typing import (
     Type, Tuple, Optional, Any, Union, List, Set,
-    Dict, Sequence, get_type_hints,
-    MutableSet, TypeVar, FrozenSet, Mapping, Callable, NamedTuple, ForwardRef, NewType,
+    Dict, Sequence, MutableSet, TypeVar, FrozenSet, Mapping, ForwardRef, NewType,
 )
 
 import inspect
@@ -11,6 +11,8 @@ import typing_inspect as insp
 from pyrsistent import pmap, pvector
 from pyrsistent import typing as pyt
 
+from .type_info import get_type_attribute_info, AttrInfo, NoneType
+
 from ..compat import Literal
 from ..definitions import OverridesT
 from ..utils import is_named_tuple, clone_schema_node, get_global_name_overrider
@@ -24,19 +26,13 @@ from .. import interface as iface
 T = TypeVar('T')
 MemoType = TypeVar('MemoType')
 
-NoneType = type(None)
-
-
-OverrideT = Union[
-    # flag override
-    flags._Flag,
-    # new type extension
-    TypeExtension,
-    Union[
-        Mapping[property, str],         # overrides syntax for NamedTuples
-        Mapping[Tuple[Type, str], str]  # overrides syntax for dataclasses and init-based hints
-    ],
-]
+OverrideT = (   flags._Flag     # flag override
+            |   TypeExtension   # new type extension
+            |   Union[
+                    Mapping[property, str],         # overrides syntax for NamedTuples
+                    Mapping[Tuple[Type, str], str]  # overrides syntax for dataclasses and init-based hints
+                ]
+            )
 
 
 def inner_type_boundaries(typ: Type) -> Tuple:
@@ -149,6 +145,10 @@ def _maybe_node_for_subclass_based(
     return rv, memo, forward_refs
 
 
+# checks for "T1 | T2" syntax instances resulting in UnionType
+is_py_310_union = lambda instance: type(instance) is UnionType
+
+
 def _maybe_node_for_union(
     typ: Type[iface.IType],
     overrides: OverridesT,
@@ -156,15 +156,15 @@ def _maybe_node_for_union(
     forward_refs: ForwardRefs,
     supported_type=frozenset({}),
     supported_origin=frozenset({
-        Union,
+        Union, UnionType
     })
 ) -> Tuple[Optional[schema.nodes.SchemaNode], MemoType, ForwardRefs]:
     """ Handles cases where typ is a Union, including the special
     case of Optional[Any], which is in essence Union[None, T]
     where T is either unknown Any or a concrete type.
     """
-    if typ in supported_type or get_origin_39(typ) in supported_origin:
-        variants = inner_type_boundaries(typ)
+    if typ in supported_type or get_origin_39(typ) in supported_origin or is_py_310_union(typ):
+        variants = inner_type_boundaries(typ) or typ.__args__  # 'or x' assumes Python310 union
         if variants in ((NoneType, Any), (Any, NoneType)):
             # Case for Optional[Any] and Union[None, Any] notations
             rv = schema.nodes.SchemaNode(
@@ -440,15 +440,23 @@ def _maybe_node_for_dict(
     if typ in supported_type or get_origin_39(typ) in supported_origin or are_generic_bases_match(generic_bases, supported_origin):
         schema_node_type = schema.nodes.PMapSchema if is_pmap(typ) else schema.nodes.SchemaNode
 
-        if generic_bases:
-            # python 3.9 args
-            key_type, value_type = typ.__args__
-        else:
+        def maybe_non_specified_arguments(mappingType):
             try:
-                key_type, value_type = insp.get_args(typ)
+                kt, kv = insp.get_args(mappingType)
             except ValueError:
                 # Mapping doesn't provide key/value types
-                key_type, value_type = Any, Any
+                kt, kv = Any, Any
+            return kt, kv
+
+        if generic_bases:
+            # python 3.9 args
+            try:
+                key_type, value_type = typ.__args__
+            except AttributeError:
+                # PMap without clarifying type arguments will cause this branch
+                key_type, value_type = maybe_non_specified_arguments(typ)
+        else:
+            key_type, value_type = maybe_non_specified_arguments(typ)
 
         key_node,   memo, forward_refs = decide_node_type(key_type, overrides, memo, forward_refs)
         value_node, memo, forward_refs = decide_node_type(value_type, overrides, memo, forward_refs)
@@ -457,18 +465,6 @@ def _maybe_node_for_dict(
     return rv, memo, forward_refs
 
 
-class AttrInfo(NamedTuple):
-    name: str
-    resolved_type: Type
-    raw_type: Union[Type, ForwardRef]
-
-
-def _type_hints_getter(typ: Type) -> Sequence[AttrInfo]:
-    raw = getattr(typ, '__annotations__', {})
-    existing_only = lambda x: x[1] is not NoneType
-    return [AttrInfo(name, t, raw.get(name, t)) for name, t in filter(existing_only, get_type_hints(typ).items())]
-
-
 def _maybe_node_for_user_type(
     typ: Type[iface.IType],
     overrides: OverridesT,
@@ -493,7 +489,7 @@ def _maybe_node_for_user_type(
         type_var_to_type = pmap(zip(generic_vars_ordered, bound_type_args))
         # resolve type hints
         attribute_hints = [(field_name, type_var_to_type[type_var])
-                           for field_name, type_var in ((x, raw_type) for x, _resolved_type, raw_type in _type_hints_getter(hints_source))]
+                           for field_name, type_var in ((x, raw_type) for x, _resolved_type, raw_type in get_type_attribute_info(hints_source))]
         # Generic types should not have default values
         defaults_source = lambda: ()
         # Overrides should be the same as class-based ones, as Generics are not NamedTuple classes,
@@ -514,7 +510,7 @@ def _maybe_node_for_user_type(
 
     elif is_named_tuple(typ):
         hints_source = typ
-        attribute_hints = [(x, raw_type) for x, y, raw_type in _type_hints_getter(hints_source)]
+        attribute_hints = [(x, raw_type) for x, y, raw_type in get_type_attribute_info(hints_source)]
         get_override_identifier = lambda x: getattr(typ, x)
         defaults_source = typ.__new__
 
@@ -535,7 +531,7 @@ def _maybe_node_for_user_type(
     else:
         # use init-based types
         hints_source = typ.__init__
-        attribute_hints = [(x, raw_type) for x, y, raw_type in _type_hints_getter(hints_source)]
+        attribute_hints = [(x, raw_type) for x, y, raw_type in get_type_attribute_info(hints_source)]
         get_override_identifier = lambda x: (typ, x)
         defaults_source = typ.__init__
 
